/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel23-tictactoe-core.
 *
 * ipspiel23-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel23-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel23-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel23.tictactoe.core.domain.impl;

import de.fhdw.gaming.core.domain.AbstractPlayer;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeFieldState;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToePlayer;

/**
 * Implements {@link TicTacToePlayer}.
 */
final class TicTacToePlayerImpl extends AbstractPlayer<TicTacToePlayer> implements TicTacToePlayer {

    /**
     * {@code true} if this player uses crosses, or {@code false} if she uses noughts.
     */
    private final boolean usingCrosses;

    /**
     * Creates a Tic Tac Toe player.
     *
     * @param name         The name of the player.
     * @param usingCrosses {@code true} if this player uses crosses, or {@code false} if she uses noughts..
     */
    TicTacToePlayerImpl(final String name, final boolean usingCrosses) {
        super(name);
        this.usingCrosses = usingCrosses;
    }

    /**
     * Creates a Tic Tac Toe player.
     *
     * @param source The {@link TicTacToePlayer} to copy.
     */
    TicTacToePlayerImpl(final TicTacToePlayer source) {
        super(source);
        this.usingCrosses = source.isUsingCrosses();
    }

    @Override
    public String toString() {
        return String.format(
                "TicTacToePlayer[name=%s, type=%s, state=%s, outcome=%s]",
                this.getName(),
                this.usingCrosses ? TicTacToeFieldState.CROSS.toString() : TicTacToeFieldState.NOUGHT.toString(),
                this.getState(),
                this.getOutcome());
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof TicTacToePlayerImpl) {
            final TicTacToePlayerImpl other = (TicTacToePlayerImpl) obj;
            return super.equals(obj) && this.usingCrosses == other.usingCrosses;
        }
        return false;
    }

    @Override
    public int hashCode() {
        return super.hashCode() ^ Boolean.hashCode(this.usingCrosses);
    }

    @Override
    public boolean isUsingCrosses() {
        return this.usingCrosses;
    }

    @Override
    public TicTacToePlayer deepCopy() {
        return new TicTacToePlayerImpl(this);
    }
}
